<?php

namespace Drupal\theme_change\Theme;

use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Theme\ThemeNegotiatorInterface;

class ThemeChangeswitcherNegotiator implements ThemeNegotiatorInterface {

  public function applies(RouteMatchInterface $route_match) {
    $wildcard = FALSE;
    $applies = FALSE;
    // Get Current Path.
    $current_path = \Drupal::service('path.current')->getPath();
    // Get Alias of path.
    $pathAlias = \Drupal::service('path.alias_manager')->getAliasByPath($current_path);
    // Get Current Route name.
    $route_name = \Drupal::routeMatch()->getRouteName();
    // Get All route names.
    $routes = $this->_get_route_names();
    // Get all path values.
    $paths = $this->_get_path_values();
    // Get all wildcard path values.
    $wildcard_paths = $this->_get_path_wildcard_values();
    foreach ($wildcard_paths as $wildcard_path) {
      $path_matches = \Drupal::service('path.matcher')->matchPath($current_path, $wildcard_path);
      if ($path_matches) {
        $wildcard = TRUE;
      }
    }
    if ($routes && in_array($route_name, $routes)) {
      $applies = TRUE;
    }
    else if ($paths && (in_array($current_path, $paths) || in_array($pathAlias, $paths))) {
      $applies = TRUE;
    }
    else if ($wildcard) {
      $applies = TRUE;
    }
    // Use this theme negotiator.
    return $applies;
  }

  /**
   * {@inheritdoc}
   */
  public function determineActiveTheme(RouteMatchInterface $route_match) {
    $current_path = \Drupal::service('path.current')->getPath();
    // Get Alias of path.
    $pathAlias = \Drupal::service('path.alias_manager')->getAliasByPath($current_path);
    // Get Current Route name.
    $route_name = \Drupal::routeMatch()->getRouteName();
    // Get All route names.
    $routes = $this->_get_route_names();
    // Get all path values.
    $paths = $this->_get_path_values(); // Get all wildcard path values.
    $wildcard_paths = $this->_get_path_wildcard_values();
    foreach ($wildcard_paths as $wildcard_path) {
      $path_matches = \Drupal::service('path.matcher')->matchPath($current_path, $wildcard_path);
      if ($path_matches) {
        $wildcard = TRUE;
        $wildcard_theme = $wildcard_path;
        break;
      }
    }
    if ($routes && in_array($route_name, $routes)) {
      $theme = $this->_get_theme($route_name);
      return $theme;
    }
    else if ($paths && (in_array($current_path, $paths) || in_array($pathAlias, $paths))) {
      if ($current_path) {
        $theme = $this->_get_theme($current_path);
        return $theme;
      }
      elseif ($pathAlias) {
        $theme = $this->_get_theme($current_path);
        return $theme;
      }
    }
    else if ($wildcard && $wildcard_theme) {
      $theme = $this->_get_theme($wildcard_theme);
      return $theme;
    }
  }

  /**
   * Get all route name(s).
   */
  public function _get_route_names() {
    $routes = [];
    $route_check = \Drupal::entityQuery('theme_change')
        ->execute();
    $entity = \Drupal::entityTypeManager()->getStorage('theme_change');
    foreach ($route_check as $value) {
      $entity_load = $entity->load($value);
      if ($entity_load->get_route()) {
        $routes[$entity_load->get_route()] = $entity_load->get_route();
      }
    }
    return isset($routes) ? $routes : null;
  }

  /**
   * Get all path value(s).
   */
  public function _get_path_values() {
    $paths = [];
    $path_check = \Drupal::entityQuery('theme_change')
        ->execute();
    $entity = \Drupal::entityTypeManager()->getStorage('theme_change');
    foreach ($path_check as $value) {
      $entity_load = $entity->load($value);
      if ($entity_load->get_path()) {
        $paths[$entity_load->get_path()] = $entity_load->get_path();
      }
    }
    return isset($paths) ? $paths : null;
  }

  /**
   * Get all wildcard path value(s).
   */
  public function _get_path_wildcard_values() {
    $paths = [];
    $path_check = \Drupal::entityQuery('theme_change')
        ->execute();
    $entity = \Drupal::entityTypeManager()->getStorage('theme_change');
    foreach ($path_check as $value) {
      $entity_load = $entity->load($value);
      $path = $entity_load->get_path();
      if ($path) {
        if ((strpos($path, '/*') !== false) || (strpos($path, '/%') !== false)) {
          $paths[$entity_load->get_path()] = $entity_load->get_path();
        }
      }
    }
    return $paths;
  }

  /**
   * Get theme based on path/route.
   */
  public function _get_theme($value) {
    $path_check = \Drupal::entityQuery('theme_change')
        ->condition('path', $value)
        ->execute();
    if ($path_check) {
      $entity = \Drupal::entityTypeManager()->getStorage('theme_change');
      $entity_load = $entity->load(key($path_check));
      $path = $entity_load->get_theme();
      return $path;
    }
    else {
      $route_check = \Drupal::entityQuery('theme_change')
          ->condition('route', $value)
          ->execute();
      if ($route_check) {
        $entity = \Drupal::entityTypeManager()->getStorage('theme_change');
        $entity_load = $entity->load(key($route_check));
        $path = $entity_load->get_theme();
        return $path;
      }
    }
  }

}
